@@ -2,30 +2,18 @@
 # Copyright (c) 2016, Emmanuel Bouaziz <ebouaziz@free.fr>
 # All rights reserved.
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the Neotion nor the names of its contributors may
-#       be used to endorse or promote products derived from this software
-#       without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL NEOTION BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
-# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """SPI support for PyFdti"""
 
+#pylint: disable-msg=too-many-arguments
+#pylint: disable-msg=too-many-locals
+#pylint: disable-msg=too-many-branches
+#pylint: disable-msg=too-many-statements
+#pylint: disable-msg=too-many-instance-attributes
+#pylint: disable-msg=too-many-public-methods
+#pylint: disable-msg=invalid-name
+
 from logging import getLogger
 from struct import calcsize as scalc, pack as spack, unpack as sunpack
 from threading import Lock
@@ -33,13 +21,6 @@ from typing import Any, Iterable, Mapping, Optional, Set, Union
 from usb.core import Device as UsbDevice
 from .ftdi import Ftdi, FtdiError
 
-#pylint: disable-msg=too-many-arguments
-#pylint: disable-msg=too-many-locals
-#pylint: disable-msg=too-many-branches
-#pylint: disable-msg=too-many-statements
-#pylint: disable-msg=too-many-instance-attributes
-#pylint: disable-msg=invalid-name
-
 
 class SpiIOError(FtdiError):
     """SPI I/O error"""
@@ -461,12 +442,23 @@ class SpiController:
             if not self._wide_port:
                 self._set_gpio_direction(8, io_out & 0xFF, io_dir & 0xFF)
 
+    def close(self, freeze: bool = False) -> None:
+        """Close the FTDI interface.
+
+           :param freeze: if set, FTDI port is not reset to its default
+                          state on close.
+        """
+        with self._lock:
+            if self._ftdi.is_connected:
+                self._ftdi.close(freeze)
+        self._frequency = 0.0
+
     def terminate(self) -> None:
         """Close the FTDI interface.
+
+           :note: deprecated API, use close()
         """
-        if self._ftdi:
-            self._ftdi.close()
-        self._frequency = 0.0
+        self.close()
 
     def get_port(self, cs: int, freq: Optional[float] = None,
                  mode: int = 0) -> SpiPort:
