@@ -1,31 +1,12 @@
 # Copyright (c) 2017-2020, Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the Neotion nor the names of its contributors may
-#       be used to endorse or promote products derived from this software
-#       without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL NEOTION BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
-# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """MPSSE command debug tracer."""
 
 #pylint: disable-msg=missing-docstring
+#pylint: disable-msg=too-many-instance-attributes
 
 from binascii import hexlify
 from collections import deque
@@ -65,9 +46,9 @@ class FtdiMpsseTracer:
         iface -= 1
         try:
             self._engines[iface]
-        except IndexError:
+        except IndexError as exc:
             raise ValueError('No MPSSE engine available on interface %d' %
-                             iface)
+                             iface) from exc
         if not self._engines[iface]:
             self._engines[iface] = FtdiMpsseEngine(iface)
         return self._engines[iface]
@@ -126,7 +107,7 @@ class FtdiMpsseEngine:
                 try:
                     self._cmd_decoded = cmd_decoder()
                 except AttributeError as exc:
-                    raise ValueError(str(exc))
+                    raise ValueError(str(exc)) from exc
                 if len(self._expect_resp) > rdepth:
                     self._last_codes.append(code)
                 if self._cmd_decoded:
